@@ -1,3 +1,13 @@
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+
+
 use crate::rustc::hir;
 use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
 use crate::rustc::{declare_tool_lint, lint_array};
@@ -98,8 +108,10 @@ impl ExcessivePrecision {
     }
 }
 
-/// Should we exclude the float because it has a .0 suffix
+#[allow(clippy::doc_markdown)]
+/// Should we exclude the float because it has a `.0` or `.` suffix
 /// Ex 1_000_000_000.0
+/// Ex 1_000_000_000.
 fn dot_zero_exclusion(s: &str) -> bool {
     if let Some(after_dec) = s.split('.').nth(1) {
         let mut decpart = after_dec
@@ -108,7 +120,8 @@ fn dot_zero_exclusion(s: &str) -> bool {
 
         match decpart.next() {
             Some('0') => decpart.count() == 0,
-            _ => false,
+            Some(_) => false,
+            None => true,
         }
     } else {
         false
@@ -124,10 +137,10 @@ fn max_digits(fty: FloatTy) -> u32 {
 
 /// Counts the digits excluding leading zeros
 fn count_digits(s: &str) -> usize {
-    // Note that s does not contain the f32/64 suffix
+    // Note that s does not contain the f32/64 suffix, and underscores have been stripped
     s.chars()
-        .filter(|c| *c != '-' || *c != '.')
-        .take_while(|c| *c != 'e' || *c != 'E')
+        .filter(|c| *c != '-' && *c != '.')
+        .take_while(|c| *c != 'e' && *c != 'E')
         .fold(0, |count, c| {
             // leading zeros
             if c == '0' && count == 0 {
