@@ -1,3 +1,13 @@
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+
+
 //! lint on inherent implementations
 
 use crate::rustc::hir::*;
@@ -46,7 +56,7 @@ pub struct Pass {
 
 impl Default for Pass {
     fn default() -> Self {
-        Pass { impls: FxHashMap::default() }
+        Self { impls: FxHashMap::default() }
     }
 }
 
