@@ -1,3 +1,13 @@
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+
+
 use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
 use crate::rustc::{declare_tool_lint, lint_array};
 use if_chain::if_chain;
@@ -8,6 +18,7 @@ use crate::syntax::ptr::P;
 
 use crate::utils::{match_def_path, match_type, span_lint_and_then};
 use crate::utils::paths::*;
+use crate::rustc_errors::Applicability;
 
 /// **What it does:** Checks for expressions that could be replaced by the question mark operator
 ///
@@ -34,15 +45,15 @@ declare_clippy_lint!{
 }
 
 #[derive(Copy, Clone)]
-pub struct QuestionMarkPass;
+pub struct Pass;
 
-impl LintPass for QuestionMarkPass {
+impl LintPass for Pass {
     fn get_lints(&self) -> LintArray {
         lint_array!(QUESTION_MARK)
     }
 }
 
-impl QuestionMarkPass {
+impl Pass {
     /// Check if the given expression on the given context matches the following structure:
     ///
     /// ```ignore
@@ -70,10 +81,11 @@ impl QuestionMarkPass {
                     |db| {
                         let receiver_str = &Sugg::hir(cx, subject, "..");
 
-                        db.span_suggestion(
+                        db.span_suggestion_with_applicability(
                             expr.span,
                             "replace_it_with",
                             format!("{}?;", receiver_str),
+                            Applicability::MachineApplicable, // snippet
                         );
                     }
                 )
@@ -133,7 +145,7 @@ impl QuestionMarkPass {
     }
 }
 
-impl<'a, 'tcx> LateLintPass<'a, 'tcx> for QuestionMarkPass {
+impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Pass {
     fn check_expr(&mut self, cx: &LateContext<'a, 'tcx>, expr: &'tcx Expr) {
         Self::check_is_none_and_early_return_none(cx, expr);
     }
