@@ -1,3 +1,13 @@
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+
+
 use crate::rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
 use crate::rustc::{declare_tool_lint, lint_array};
 use if_chain::if_chain;
@@ -7,6 +17,7 @@ use crate::syntax::source_map::Spanned;
 use crate::utils::{is_integer_literal, paths, snippet, span_lint, span_lint_and_then, snippet_opt};
 use crate::utils::{get_trait_def_id, higher, implements_trait, SpanlessEq};
 use crate::utils::sugg::Sugg;
+use crate::rustc_errors::Applicability;
 
 /// **What it does:** Checks for calling `.step_by(0)` on iterators,
 /// which never terminates.
@@ -150,13 +161,19 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Pass {
                         let end = Sugg::hir(cx, y, "y");
                         if let Some(is_wrapped) = &snippet_opt(cx, expr.span) {
                             if is_wrapped.starts_with('(') && is_wrapped.ends_with(')') {
-                                db.span_suggestion(expr.span,
-                                           "use",
-                                           format!("({}..={})", start, end));
+                                db.span_suggestion_with_applicability(
+                                    expr.span,
+                                    "use",
+                                    format!("({}..={})", start, end),
+                                    Applicability::MaybeIncorrect,
+                                );
                             } else {
-                                db.span_suggestion(expr.span,
-                                           "use",
-                                           format!("{}..={}", start, end));
+                                db.span_suggestion_with_applicability(
+                                    expr.span,
+                                    "use",
+                                    format!("{}..={}", start, end),
+                                    Applicability::MachineApplicable, // snippet
+                                );
                             }
                         }
                     },
@@ -177,9 +194,12 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Pass {
                     |db| {
                         let start = start.map_or(String::new(), |x| Sugg::hir(cx, x, "x").to_string());
                         let end = Sugg::hir(cx, y, "y");
-                        db.span_suggestion(expr.span,
-                                           "use",
-                                           format!("{}..{}", start, end));
+                        db.span_suggestion_with_applicability(
+                            expr.span,
+                            "use",
+                            format!("{}..{}", start, end),
+                            Applicability::MachineApplicable, // snippet
+                        );
                     },
                 );
             }
