@@ -1,3 +1,13 @@
+// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
+// file at the top-level directory of this distribution.
+//
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
+
+
 //! Read configurations files.
 
 #![deny(clippy::missing_docs_in_private_items)]
@@ -148,7 +158,7 @@ define_Conf! {
 }
 
 impl Default for Conf {
-    fn default() -> Conf {
+    fn default() -> Self {
         toml::from_str("").expect("we never error on empty config files")
     }
 }
