@@ -8,18 +8,14 @@ use std::mem;
 
 use super::*;
 
-use tls::MemoryExt;
-
-use super::memory::MemoryKind;
-
 pub trait EvalContextExt<'tcx, 'mir> {
     /// Emulate calling a foreign item, fail if the item is not supported.
     /// This function will handle `goto_block` if needed.
     fn emulate_foreign_item(
         &mut self,
         def_id: DefId,
-        args: &[OpTy<'tcx>],
-        dest: PlaceTy<'tcx>,
+        args: &[OpTy<'tcx, Borrow>],
+        dest: PlaceTy<'tcx, Borrow>,
         ret: mir::BasicBlock,
     ) -> EvalResult<'tcx>;
 
@@ -32,28 +28,28 @@ pub trait EvalContextExt<'tcx, 'mir> {
     fn emulate_missing_fn(
         &mut self,
         path: String,
-        args: &[OpTy<'tcx>],
-        dest: Option<PlaceTy<'tcx>>,
+        args: &[OpTy<'tcx, Borrow>],
+        dest: Option<PlaceTy<'tcx, Borrow>>,
         ret: Option<mir::BasicBlock>,
     ) -> EvalResult<'tcx>;
 
     fn find_fn(
         &mut self,
         instance: ty::Instance<'tcx>,
-        args: &[OpTy<'tcx>],
-        dest: Option<PlaceTy<'tcx>>,
+        args: &[OpTy<'tcx, Borrow>],
+        dest: Option<PlaceTy<'tcx, Borrow>>,
         ret: Option<mir::BasicBlock>,
     ) -> EvalResult<'tcx, Option<&'mir mir::Mir<'tcx>>>;
 
-    fn write_null(&mut self, dest: PlaceTy<'tcx>) -> EvalResult<'tcx>;
+    fn write_null(&mut self, dest: PlaceTy<'tcx, Borrow>) -> EvalResult<'tcx>;
 }
 
-impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, 'mir, 'tcx, super::Evaluator<'tcx>> {
+impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for super::MiriEvalContext<'a, 'mir, 'tcx> {
     fn find_fn(
         &mut self,
         instance: ty::Instance<'tcx>,
-        args: &[OpTy<'tcx>],
-        dest: Option<PlaceTy<'tcx>>,
+        args: &[OpTy<'tcx, Borrow>],
+        dest: Option<PlaceTy<'tcx, Borrow>>,
         ret: Option<mir::BasicBlock>,
     ) -> EvalResult<'tcx, Option<&'mir mir::Mir<'tcx>>> {
         trace!("eval_fn_call: {:#?}, {:?}", instance, dest.map(|place| *place));
@@ -112,8 +108,8 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
     fn emulate_foreign_item(
         &mut self,
         def_id: DefId,
-        args: &[OpTy<'tcx>],
-        dest: PlaceTy<'tcx>,
+        args: &[OpTy<'tcx, Borrow>],
+        dest: PlaceTy<'tcx, Borrow>,
         ret: mir::BasicBlock,
     ) -> EvalResult<'tcx> {
         let attrs = self.tcx.get_attrs(def_id);
@@ -122,6 +118,10 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
             None => self.tcx.item_name(def_id).as_str(),
         };
 
+        // All these functions take raw pointers, so if we access memory directly
+        // (as opposed to through a place), we have to remember to erase any tag
+        // that might still hang around!
+
         match &link_name[..] {
             "malloc" => {
                 let size = self.read_scalar(args[0])?.to_usize(&self)?;
@@ -129,18 +129,18 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                     self.write_null(dest)?;
                 } else {
                     let align = self.tcx.data_layout.pointer_align;
-                    let ptr = self.memory.allocate(Size::from_bytes(size), align, MemoryKind::C.into())?;
+                    let ptr = self.memory.allocate(Size::from_bytes(size), align, MiriMemoryKind::C.into())?;
                     self.write_scalar(Scalar::Ptr(ptr), dest)?;
                 }
             }
 
             "free" => {
-                let ptr = self.read_scalar(args[0])?.not_undef()?;
+                let ptr = self.read_scalar(args[0])?.not_undef()?.erase_tag(); // raw ptr operation, no tag
                 if !ptr.is_null() {
                     self.memory.deallocate(
-                        ptr.to_ptr()?,
+                        ptr.to_ptr()?.with_default_tag(),
                         None,
-                        MemoryKind::C.into(),
+                        MiriMemoryKind::C.into(),
                     )?;
                 }
             }
@@ -156,7 +156,7 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                 }
                 let ptr = self.memory.allocate(Size::from_bytes(size),
                                                Align::from_bytes(align, align).unwrap(),
-                                               MemoryKind::Rust.into())?;
+                                               MiriMemoryKind::Rust.into())?;
                 self.write_scalar(Scalar::Ptr(ptr), dest)?;
             }
             "__rust_alloc_zeroed" => {
@@ -170,12 +170,12 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                 }
                 let ptr = self.memory.allocate(Size::from_bytes(size),
                                                Align::from_bytes(align, align).unwrap(),
-                                               MemoryKind::Rust.into())?;
+                                               MiriMemoryKind::Rust.into())?;
                 self.memory.write_repeat(ptr.into(), 0, Size::from_bytes(size))?;
                 self.write_scalar(Scalar::Ptr(ptr), dest)?;
             }
             "__rust_dealloc" => {
-                let ptr = self.read_scalar(args[0])?.to_ptr()?;
+                let ptr = self.read_scalar(args[0])?.to_ptr()?.erase_tag(); // raw ptr operation, no tag
                 let old_size = self.read_scalar(args[1])?.to_usize(&self)?;
                 let align = self.read_scalar(args[2])?.to_usize(&self)?;
                 if old_size == 0 {
@@ -185,13 +185,13 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                     return err!(HeapAllocNonPowerOfTwoAlignment(align));
                 }
                 self.memory.deallocate(
-                    ptr,
+                    ptr.with_default_tag(),
                     Some((Size::from_bytes(old_size), Align::from_bytes(align, align).unwrap())),
-                    MemoryKind::Rust.into(),
+                    MiriMemoryKind::Rust.into(),
                 )?;
             }
             "__rust_realloc" => {
-                let ptr = self.read_scalar(args[0])?.to_ptr()?;
+                let ptr = self.read_scalar(args[0])?.to_ptr()?.erase_tag(); // raw ptr operation, no tag
                 let old_size = self.read_scalar(args[1])?.to_usize(&self)?;
                 let align = self.read_scalar(args[2])?.to_usize(&self)?;
                 let new_size = self.read_scalar(args[3])?.to_usize(&self)?;
@@ -202,12 +202,12 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                     return err!(HeapAllocNonPowerOfTwoAlignment(align));
                 }
                 let new_ptr = self.memory.reallocate(
-                    ptr,
+                    ptr.with_default_tag(),
                     Size::from_bytes(old_size),
                     Align::from_bytes(align, align).unwrap(),
                     Size::from_bytes(new_size),
                     Align::from_bytes(align, align).unwrap(),
-                    MemoryKind::Rust.into(),
+                    MiriMemoryKind::Rust.into(),
                 )?;
                 self.write_scalar(Scalar::Ptr(new_ptr), dest)?;
             }
@@ -234,8 +234,8 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
 
             "dlsym" => {
                 let _handle = self.read_scalar(args[0])?;
-                let symbol = self.read_scalar(args[1])?.to_ptr()?;
-                let symbol_name = self.memory.read_c_str(symbol)?;
+                let symbol = self.read_scalar(args[1])?.to_ptr()?.erase_tag();
+                let symbol_name = self.memory.read_c_str(symbol.with_default_tag())?;
                 let err = format!("bad c unicode symbol: {:?}", symbol_name);
                 let symbol_name = ::std::str::from_utf8(symbol_name).unwrap_or(&err);
                 return err!(Unimplemented(format!(
@@ -256,12 +256,12 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                 // Now we make a function call.  TODO: Consider making this re-usable?  EvalContext::step does sth. similar for the TLS dtors,
                 // and of course eval_main.
                 let mir = self.load_mir(f_instance.def)?;
-                let closure_dest = Place::null(&self);
+                let ret_place = MPlaceTy::dangling(self.layout_of(self.tcx.mk_unit())?, &self).into();
                 self.push_stack_frame(
                     f_instance,
                     mir.span,
                     mir,
-                    closure_dest,
+                    Some(ret_place),
                     StackPopCleanup::Goto(Some(ret)), // directly return to caller
                 )?;
                 let mut args = self.frame().mir.args_iter();
@@ -288,13 +288,13 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                 return err!(MachineError("the evaluated program panicked".to_string())),
 
             "memcmp" => {
-                let left = self.read_scalar(args[0])?.not_undef()?;
-                let right = self.read_scalar(args[1])?.not_undef()?;
+                let left = self.read_scalar(args[0])?.not_undef()?.erase_tag(); // raw ptr operation
+                let right = self.read_scalar(args[1])?.not_undef()?.erase_tag(); // raw ptr operation
                 let n = Size::from_bytes(self.read_scalar(args[2])?.to_usize(&self)?);
 
                 let result = {
-                    let left_bytes = self.memory.read_bytes(left, n)?;
-                    let right_bytes = self.memory.read_bytes(right, n)?;
+                    let left_bytes = self.memory.read_bytes(left.with_default_tag(), n)?;
+                    let right_bytes = self.memory.read_bytes(right.with_default_tag(), n)?;
 
                     use std::cmp::Ordering::*;
                     match left_bytes.cmp(right_bytes) {
@@ -311,12 +311,12 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
             }
 
             "memrchr" => {
-                let ptr = self.read_scalar(args[0])?.not_undef()?;
+                let ptr = self.read_scalar(args[0])?.not_undef()?.erase_tag(); // raw ptr operation
+                let ptr = ptr.with_default_tag();
                 let val = self.read_scalar(args[1])?.to_bytes()? as u8;
                 let num = self.read_scalar(args[2])?.to_usize(&self)?;
-                if let Some(idx) = self.memory.read_bytes(ptr, Size::from_bytes(num))?.iter().rev().position(
-                    |&c| c == val,
-                )
+                if let Some(idx) = self.memory.read_bytes(ptr, Size::from_bytes(num))?
+                    .iter().rev().position(|&c| c == val)
                 {
                     let new_ptr = ptr.ptr_offset(Size::from_bytes(num - idx as u64 - 1), &self)?;
                     self.write_scalar(new_ptr, dest)?;
@@ -326,7 +326,8 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
             }
 
             "memchr" => {
-                let ptr = self.read_scalar(args[0])?.not_undef()?;
+                let ptr = self.read_scalar(args[0])?.not_undef()?.erase_tag(); // raw ptr operation
+                let ptr = ptr.with_default_tag();
                 let val = self.read_scalar(args[1])?.to_bytes()? as u8;
                 let num = self.read_scalar(args[2])?.to_usize(&self)?;
                 if let Some(idx) = self.memory.read_bytes(ptr, Size::from_bytes(num))?.iter().position(
@@ -342,8 +343,8 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
 
             "getenv" => {
                 let result = {
-                    let name_ptr = self.read_scalar(args[0])?.to_ptr()?;
-                    let name = self.memory.read_c_str(name_ptr)?;
+                    let name_ptr = self.read_scalar(args[0])?.to_ptr()?.erase_tag(); // raw ptr operation
+                    let name = self.memory.read_c_str(name_ptr.with_default_tag())?;
                     match self.machine.env_vars.get(name) {
                         Some(&var) => Scalar::Ptr(var),
                         None => Scalar::ptr_null(*self.tcx),
@@ -355,9 +356,9 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
             "unsetenv" => {
                 let mut success = None;
                 {
-                    let name_ptr = self.read_scalar(args[0])?.not_undef()?;
+                    let name_ptr = self.read_scalar(args[0])?.not_undef()?.erase_tag(); // raw ptr operation
                     if !name_ptr.is_null() {
-                        let name = self.memory.read_c_str(name_ptr.to_ptr()?)?;
+                        let name = self.memory.read_c_str(name_ptr.to_ptr()?.with_default_tag())?;
                         if !name.is_empty() && !name.contains(&b'=') {
                             success = Some(self.machine.env_vars.remove(name));
                         }
@@ -365,7 +366,7 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                 }
                 if let Some(old) = success {
                     if let Some(var) = old {
-                        self.memory.deallocate(var, None, MemoryKind::Env.into())?;
+                        self.memory.deallocate(var, None, MiriMemoryKind::Env.into())?;
                     }
                     self.write_null(dest)?;
                 } else {
@@ -376,11 +377,11 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
             "setenv" => {
                 let mut new = None;
                 {
-                    let name_ptr = self.read_scalar(args[0])?.not_undef()?;
-                    let value_ptr = self.read_scalar(args[1])?.to_ptr()?;
-                    let value = self.memory.read_c_str(value_ptr)?;
+                    let name_ptr = self.read_scalar(args[0])?.not_undef()?.erase_tag(); // raw ptr operation
+                    let value_ptr = self.read_scalar(args[1])?.to_ptr()?.erase_tag(); // raw ptr operation
+                    let value = self.memory.read_c_str(value_ptr.with_default_tag())?;
                     if !name_ptr.is_null() {
-                        let name = self.memory.read_c_str(name_ptr.to_ptr()?)?;
+                        let name = self.memory.read_c_str(name_ptr.to_ptr()?.with_default_tag())?;
                         if !name.is_empty() && !name.contains(&b'=') {
                             new = Some((name.to_owned(), value.to_owned()));
                         }
@@ -391,7 +392,7 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                     let value_copy = self.memory.allocate(
                         Size::from_bytes((value.len() + 1) as u64),
                         Align::from_bytes(1, 1).unwrap(),
-                        MemoryKind::Env.into(),
+                        MiriMemoryKind::Env.into(),
                     )?;
                     self.memory.write_bytes(value_copy.into(), &value)?;
                     let trailing_zero_ptr = value_copy.offset(Size::from_bytes(value.len() as u64), &self)?.into();
@@ -401,7 +402,7 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                         value_copy,
                     )
                     {
-                        self.memory.deallocate(var, None, MemoryKind::Env.into())?;
+                        self.memory.deallocate(var, None, MiriMemoryKind::Env.into())?;
                     }
                     self.write_null(dest)?;
                 } else {
@@ -411,14 +412,14 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
 
             "write" => {
                 let fd = self.read_scalar(args[0])?.to_bytes()?;
-                let buf = self.read_scalar(args[1])?.not_undef()?;
+                let buf = self.read_scalar(args[1])?.not_undef()?.erase_tag();
                 let n = self.read_scalar(args[2])?.to_bytes()? as u64;
                 trace!("Called write({:?}, {:?}, {:?})", fd, buf, n);
                 let result = if fd == 1 || fd == 2 {
                     // stdout/stderr
                     use std::io::{self, Write};
 
-                    let buf_cont = self.memory.read_bytes(buf, Size::from_bytes(n))?;
+                    let buf_cont = self.memory.read_bytes(buf.with_default_tag(), Size::from_bytes(n))?;
                     let res = if fd == 1 {
                         io::stdout().write(buf_cont)
                     } else {
@@ -439,8 +440,8 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
             }
 
             "strlen" => {
-                let ptr = self.read_scalar(args[0])?.to_ptr()?;
-                let n = self.memory.read_c_str(ptr)?.len();
+                let ptr = self.read_scalar(args[0])?.to_ptr()?.erase_tag();
+                let n = self.memory.read_c_str(ptr.with_default_tag())?.len();
                 self.write_scalar(Scalar::from_uint(n as u64, dest.layout.size), dest)?;
             }
 
@@ -486,7 +487,7 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
 
             // Hook pthread calls that go to the thread-local storage memory subsystem
             "pthread_key_create" => {
-                let key_ptr = self.read_scalar(args[0])?.to_ptr()?;
+                let key_ptr = self.read_scalar(args[0])?.to_ptr()?.erase_tag(); // raw ptr operation
 
                 // Extract the function type out of the signature (that seems easier than constructing it ourselves...)
                 let dtor = match self.read_scalar(args[1])?.not_undef()? {
@@ -504,12 +505,12 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                 let key_layout = self.layout_of(key_type)?;
 
                 // Create key and write it into the memory where key_ptr wants it
-                let key = self.memory.create_tls_key(dtor) as u128;
+                let key = self.machine.tls.create_tls_key(dtor, *self.tcx) as u128;
                 if key_layout.size.bits() < 128 && key >= (1u128 << key_layout.size.bits() as u128) {
                     return err!(OutOfTls);
                 }
                 self.memory.write_scalar(
-                    key_ptr,
+                    key_ptr.with_default_tag(),
                     key_layout.align,
                     Scalar::from_uint(key, key_layout.size).into(),
                     key_layout.size,
@@ -520,19 +521,19 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
             }
             "pthread_key_delete" => {
                 let key = self.read_scalar(args[0])?.to_bytes()?;
-                self.memory.delete_tls_key(key)?;
+                self.machine.tls.delete_tls_key(key)?;
                 // Return success (0)
                 self.write_null(dest)?;
             }
             "pthread_getspecific" => {
                 let key = self.read_scalar(args[0])?.to_bytes()?;
-                let ptr = self.memory.load_tls(key)?;
+                let ptr = self.machine.tls.load_tls(key)?;
                 self.write_scalar(ptr, dest)?;
             }
             "pthread_setspecific" => {
                 let key = self.read_scalar(args[0])?.to_bytes()?;
                 let new_ptr = self.read_scalar(args[1])?.not_undef()?;
-                self.memory.store_tls(key, new_ptr)?;
+                self.machine.tls.store_tls(key, new_ptr)?;
 
                 // Return success (0)
                 self.write_null(dest)?;
@@ -607,7 +608,7 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
                 // This just creates a key; Windows does not natively support TLS dtors.
 
                 // Create key and return it
-                let key = self.memory.create_tls_key(None) as u128;
+                let key = self.machine.tls.create_tls_key(None, *self.tcx) as u128;
 
                 // Figure out how large a TLS key actually is. This is c::DWORD.
                 if dest.layout.size.bits() < 128 && key >= (1u128 << dest.layout.size.bits() as u128) {
@@ -617,13 +618,13 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
             }
             "TlsGetValue" => {
                 let key = self.read_scalar(args[0])?.to_bytes()?;
-                let ptr = self.memory.load_tls(key)?;
+                let ptr = self.machine.tls.load_tls(key)?;
                 self.write_scalar(ptr, dest)?;
             }
             "TlsSetValue" => {
                 let key = self.read_scalar(args[0])?.to_bytes()?;
                 let new_ptr = self.read_scalar(args[1])?.not_undef()?;
-                self.memory.store_tls(key, new_ptr)?;
+                self.machine.tls.store_tls(key, new_ptr)?;
 
                 // Return success (1)
                 self.write_scalar(Scalar::from_int(1, dest.layout.size), dest)?;
@@ -679,8 +680,8 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
     fn emulate_missing_fn(
         &mut self,
         path: String,
-        _args: &[OpTy<'tcx>],
-        dest: Option<PlaceTy<'tcx>>,
+        _args: &[OpTy<'tcx, Borrow>],
+        dest: Option<PlaceTy<'tcx, Borrow>>,
         ret: Option<mir::BasicBlock>,
     ) -> EvalResult<'tcx> {
         // In some cases in non-MIR libstd-mode, not having a destination is legit.  Handle these early.
@@ -728,7 +729,7 @@ impl<'a, 'mir, 'tcx: 'mir + 'a> EvalContextExt<'tcx, 'mir> for EvalContext<'a, '
         Ok(())
     }
 
-    fn write_null(&mut self, dest: PlaceTy<'tcx>) -> EvalResult<'tcx> {
+    fn write_null(&mut self, dest: PlaceTy<'tcx, Borrow>) -> EvalResult<'tcx> {
         self.write_scalar(Scalar::from_int(0, dest.layout.size), dest)
     }
 }
