@@ -13,48 +13,48 @@ extern crate rustc_mir;
 extern crate rustc_target;
 extern crate syntax;
 
-use rustc::ty::{self, TyCtxt, query::TyCtxtAt};
+use std::collections::HashMap;
+use std::borrow::Cow;
+
+use rustc::ty::{self, Ty, TyCtxt, query::TyCtxtAt};
 use rustc::ty::layout::{TyLayout, LayoutOf, Size};
 use rustc::hir::def_id::DefId;
 use rustc::mir;
 
-use syntax::ast::Mutability;
 use syntax::attr;
 
-use std::marker::PhantomData;
-use std::collections::HashMap;
 
-pub use rustc::mir::interpret::*;
 pub use rustc_mir::interpret::*;
+pub use rustc_mir::interpret::{self, AllocMap}; // resolve ambiguity
 
 mod fn_call;
 mod operator;
 mod intrinsic;
 mod helpers;
-mod memory;
 mod tls;
-mod locks;
 mod range_map;
+mod mono_hash_map;
+mod stacked_borrows;
 
 use fn_call::EvalContextExt as MissingFnsEvalContextExt;
 use operator::EvalContextExt as OperatorEvalContextExt;
 use intrinsic::EvalContextExt as IntrinsicEvalContextExt;
-use tls::EvalContextExt as TlsEvalContextExt;
-use memory::{MemoryKind as MiriMemoryKind, TlsKey, TlsEntry, MemoryData};
-use locks::LockInfo;
+use tls::{EvalContextExt as TlsEvalContextExt, TlsData};
 use range_map::RangeMap;
 use helpers::FalibleScalarExt;
+use mono_hash_map::MonoHashMap;
+use stacked_borrows::{EvalContextExt as StackedBorEvalContextExt, Borrow};
 
 pub fn create_ecx<'a, 'mir: 'a, 'tcx: 'mir>(
     tcx: TyCtxt<'a, 'tcx, 'tcx>,
     main_id: DefId,
     start_wrapper: Option<DefId>,
+    validate: bool,
 ) -> EvalResult<'tcx, EvalContext<'a, 'mir, 'tcx, Evaluator<'tcx>>> {
     let mut ecx = EvalContext::new(
         tcx.at(syntax::source_map::DUMMY_SP),
         ty::ParamEnv::reveal_all(),
-        Default::default(),
-        MemoryData::new()
+        Evaluator::new(validate),
     );
 
     let main_instance = ty::Instance::mono(ecx.tcx.tcx, main_id);
@@ -66,7 +66,6 @@ pub fn create_ecx<'a, 'mir: 'a, 'tcx: 'mir>(
                 .to_owned(),
         ));
     }
-    let ptr_size = ecx.memory.pointer_size();
 
     if let Some(start_id) = start_wrapper {
         let main_ret_ty = ecx.tcx.fn_sig(main_id).output();
@@ -88,16 +87,15 @@ pub fn create_ecx<'a, 'mir: 'a, 'tcx: 'mir>(
         }
 
         // Return value (in static memory so that it does not count as leak)
-        let size = ecx.tcx.data_layout.pointer_size;
-        let align = ecx.tcx.data_layout.pointer_align;
-        let ret_ptr = ecx.memory_mut().allocate(size, align, MiriMemoryKind::MutStatic.into())?;
+        let ret = ecx.layout_of(start_mir.return_ty())?;
+        let ret_ptr = ecx.allocate(ret, MiriMemoryKind::MutStatic.into())?;
 
         // Push our stack frame
         ecx.push_stack_frame(
             start_instance,
             start_mir.span,
             start_mir,
-            Place::from_ptr(ret_ptr, align),
+            Some(ret_ptr.into()),
             StackPopCleanup::None { cleanup: true },
         )?;
 
@@ -118,18 +116,19 @@ pub fn create_ecx<'a, 'mir: 'a, 'tcx: 'mir>(
         let foo = ecx.memory.allocate_static_bytes(b"foo\0");
         let foo_ty = ecx.tcx.mk_imm_ptr(ecx.tcx.types.u8);
         let foo_layout = ecx.layout_of(foo_ty)?;
-        let foo_place = ecx.allocate(foo_layout, MemoryKind::Stack)?; // will be interned in just a second
+        let foo_place = ecx.allocate(foo_layout, MiriMemoryKind::Env.into())?;
         ecx.write_scalar(Scalar::Ptr(foo), foo_place.into())?;
-        ecx.memory.intern_static(foo_place.to_ptr()?.alloc_id, Mutability::Immutable)?;
+        ecx.memory.mark_immutable(foo_place.to_ptr()?.alloc_id)?;
         ecx.write_scalar(foo_place.ptr, dest)?;
 
         assert!(args.next().is_none(), "start lang item has more arguments than expected");
     } else {
+        let ret_place = MPlaceTy::dangling(ecx.layout_of(tcx.mk_unit())?, &ecx).into();
         ecx.push_stack_frame(
             main_instance,
             main_mir.span,
             main_mir,
-            Place::from_scalar_ptr(Scalar::from_int(1, ptr_size).into(), ty::layout::Align::from_bytes(1, 1).unwrap()),
+            Some(ret_place),
             StackPopCleanup::None { cleanup: true },
         )?;
 
@@ -145,8 +144,9 @@ pub fn eval_main<'a, 'tcx: 'a>(
     tcx: TyCtxt<'a, 'tcx, 'tcx>,
     main_id: DefId,
     start_wrapper: Option<DefId>,
+    validate: bool,
 ) {
-    let mut ecx = create_ecx(tcx, main_id, start_wrapper).expect("Couldn't create ecx");
+    let mut ecx = create_ecx(tcx, main_id, start_wrapper, validate).expect("Couldn't create ecx");
 
     let res: EvalResult = (|| {
         ecx.run()?;
@@ -201,57 +201,138 @@ pub fn eval_main<'a, 'tcx: 'a>(
     }
 }
 
-#[derive(Clone, Default, PartialEq, Eq)]
+
+#[derive(Debug, Copy, Clone, PartialEq, Eq)]
+pub enum MiriMemoryKind {
+    /// `__rust_alloc` memory
+    Rust,
+    /// `malloc` memory
+    C,
+    /// Part of env var emulation
+    Env,
+    /// mutable statics
+    MutStatic,
+}
+
+impl Into<MemoryKind<MiriMemoryKind>> for MiriMemoryKind {
+    #[inline(always)]
+    fn into(self) -> MemoryKind<MiriMemoryKind> {
+        MemoryKind::Machine(self)
+    }
+}
+
+impl MayLeak for MiriMemoryKind {
+    #[inline(always)]
+    fn may_leak(self) -> bool {
+        use MiriMemoryKind::*;
+        match self {
+            Rust | C => false,
+            Env | MutStatic => true,
+        }
+    }
+}
+
 pub struct Evaluator<'tcx> {
     /// Environment variables set by `setenv`
     /// Miri does not expose env vars from the host to the emulated program
-    pub(crate) env_vars: HashMap<Vec<u8>, Pointer>,
+    pub(crate) env_vars: HashMap<Vec<u8>, Pointer<Borrow>>,
 
-    /// Use the lifetime
-    _dummy : PhantomData<&'tcx ()>,
+    /// TLS state
+    pub(crate) tls: TlsData<'tcx>,
+
+    /// Whether to enforce the validity invariant
+    pub(crate) validate: bool,
+
+    /// Stacked Borrows state
+    pub(crate) stacked_borrows: stacked_borrows::State,
 }
 
-impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
-    type MemoryData = memory::MemoryData<'tcx>;
-    type MemoryKinds = memory::MemoryKind;
+impl<'tcx> Evaluator<'tcx> {
+    fn new(validate: bool) -> Self {
+        Evaluator {
+            env_vars: HashMap::default(),
+            tls: TlsData::default(),
+            validate,
+            stacked_borrows: stacked_borrows::State::new(),
+        }
+    }
+}
 
-    const MUT_STATIC_KIND: Option<memory::MemoryKind> = Some(memory::MemoryKind::MutStatic);
-    const DETECT_LOOPS: bool = false;
+#[allow(dead_code)] // FIXME https://github.com/rust-lang/rust/issues/47131
+type MiriEvalContext<'a, 'mir, 'tcx> = EvalContext<'a, 'mir, 'tcx, Evaluator<'tcx>>;
+
+
+impl<'a, 'mir, 'tcx> Machine<'a, 'mir, 'tcx> for Evaluator<'tcx> {
+    type MemoryKinds = MiriMemoryKind;
+
+    type AllocExtra = stacked_borrows::Stacks;
+    type PointerTag = Borrow;
+    const ENABLE_PTR_TRACKING_HOOKS: bool = true;
+
+    type MemoryMap = MonoHashMap<AllocId, (MemoryKind<MiriMemoryKind>, Allocation<Borrow, Self::AllocExtra>)>;
+
+    const STATIC_KIND: Option<MiriMemoryKind> = Some(MiriMemoryKind::MutStatic);
+
+    fn enforce_validity(ecx: &EvalContext<'a, 'mir, 'tcx, Self>) -> bool {
+        if !ecx.machine.validate {
+            return false;
+        }
+
+        // Some functions are whitelisted until we figure out how to fix them.
+        // We walk up the stack a few frames to also cover their callees.
+        const WHITELIST: &[&str] = &[
+            // Uses mem::uninitialized
+            "std::ptr::read",
+            "std::sys::windows::mutex::Mutex::",
+        ];
+        for frame in ecx.stack().iter()
+            .rev().take(3)
+        {
+            let name = frame.instance.to_string();
+            if WHITELIST.iter().any(|white| name.starts_with(white)) {
+                return false;
+            }
+        }
+        true
+    }
 
     /// Returns Ok() when the function was handled, fail otherwise
-    fn find_fn<'a>(
+    #[inline(always)]
+    fn find_fn(
         ecx: &mut EvalContext<'a, 'mir, 'tcx, Self>,
         instance: ty::Instance<'tcx>,
-        args: &[OpTy<'tcx>],
-        dest: Option<PlaceTy<'tcx>>,
+        args: &[OpTy<'tcx, Borrow>],
+        dest: Option<PlaceTy<'tcx, Borrow>>,
         ret: Option<mir::BasicBlock>,
     ) -> EvalResult<'tcx, Option<&'mir mir::Mir<'tcx>>> {
         ecx.find_fn(instance, args, dest, ret)
     }
 
-    fn call_intrinsic<'a>(
+    #[inline(always)]
+    fn call_intrinsic(
         ecx: &mut rustc_mir::interpret::EvalContext<'a, 'mir, 'tcx, Self>,
         instance: ty::Instance<'tcx>,
-        args: &[OpTy<'tcx>],
-        dest: PlaceTy<'tcx>,
+        args: &[OpTy<'tcx, Borrow>],
+        dest: PlaceTy<'tcx, Borrow>,
     ) -> EvalResult<'tcx> {
         ecx.call_intrinsic(instance, args, dest)
     }
 
-    fn ptr_op<'a>(
+    #[inline(always)]
+    fn ptr_op(
         ecx: &rustc_mir::interpret::EvalContext<'a, 'mir, 'tcx, Self>,
         bin_op: mir::BinOp,
-        left: Scalar,
+        left: Scalar<Borrow>,
         left_layout: TyLayout<'tcx>,
-        right: Scalar,
+        right: Scalar<Borrow>,
         right_layout: TyLayout<'tcx>,
-    ) -> EvalResult<'tcx, (Scalar, bool)> {
+    ) -> EvalResult<'tcx, (Scalar<Borrow>, bool)> {
         ecx.ptr_op(bin_op, left, left_layout, right, right_layout)
     }
 
-    fn box_alloc<'a>(
+    fn box_alloc(
         ecx: &mut EvalContext<'a, 'mir, 'tcx, Self>,
-        dest: PlaceTy<'tcx>,
+        dest: PlaceTy<'tcx, Borrow>,
     ) -> EvalResult<'tcx> {
         trace!("box_alloc for {:?}", dest.layout.ty);
         // Call the `exchange_malloc` lang item
@@ -262,7 +343,7 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
             malloc,
             malloc_mir.span,
             malloc_mir,
-            *dest,
+            Some(dest),
             // Don't do anything when we are done.  The statement() function will increment
             // the old stack frame's stmt counter to the next statement, which means that when
             // exchange_malloc returns, we go on evaluating exactly where we want to be.
@@ -288,10 +369,10 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
         Ok(())
     }
 
-    fn find_foreign_static<'a>(
+    fn find_foreign_static(
         tcx: TyCtxtAt<'a, 'tcx, 'tcx>,
         def_id: DefId,
-    ) -> EvalResult<'tcx, &'tcx Allocation> {
+    ) -> EvalResult<'tcx, Cow<'tcx, Allocation<Borrow, Self::AllocExtra>>> {
         let attrs = tcx.get_attrs(def_id);
         let link_name = match attr::first_attr_value_str_by_name(&attrs, "link_name") {
             Some(name) => name.as_str(),
@@ -302,23 +383,85 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
             "__cxa_thread_atexit_impl" => {
                 // This should be all-zero, pointer-sized
                 let data = vec![0; tcx.data_layout.pointer_size.bytes() as usize];
-                let alloc = Allocation::from_bytes(&data[..], tcx.data_layout.pointer_align);
-                tcx.intern_const_alloc(alloc)
+                Allocation::from_bytes(&data[..], tcx.data_layout.pointer_align)
             }
             _ => return err!(Unimplemented(
                     format!("can't access foreign static: {}", link_name),
                 )),
         };
-        Ok(alloc)
+        Ok(Cow::Owned(alloc))
     }
 
-    fn validation_op<'a>(
-        _ecx: &mut EvalContext<'a, 'mir, 'tcx, Self>,
-        _op: ::rustc::mir::ValidationOp,
-        _operand: &::rustc::mir::ValidationOperand<'tcx, ::rustc::mir::Place<'tcx>>,
-    ) -> EvalResult<'tcx> {
-        // FIXME: prevent this from ICEing
-        //ecx.validation_op(op, operand)
+    #[inline(always)]
+    fn before_terminator(_ecx: &mut EvalContext<'a, 'mir, 'tcx, Self>) -> EvalResult<'tcx>
+    {
+        // We are not interested in detecting loops
         Ok(())
     }
+
+    fn static_with_default_tag(
+        alloc: &'_ Allocation
+    ) -> Cow<'_, Allocation<Borrow, Self::AllocExtra>> {
+        let alloc: Allocation<Borrow, Self::AllocExtra> = Allocation {
+            bytes: alloc.bytes.clone(),
+            relocations: Relocations::from_presorted(
+                alloc.relocations.iter()
+                    .map(|&(offset, ((), alloc))| (offset, (Borrow::default(), alloc)))
+                    .collect()
+            ),
+            undef_mask: alloc.undef_mask.clone(),
+            align: alloc.align,
+            mutability: alloc.mutability,
+            extra: Self::AllocExtra::default(),
+        };
+        Cow::Owned(alloc)
+    }
+
+    #[inline(always)]
+    fn memory_accessed(
+        alloc: &Allocation<Borrow, Self::AllocExtra>,
+        ptr: Pointer<Borrow>,
+        size: Size,
+        access: MemoryAccess,
+    ) -> EvalResult<'tcx> {
+        alloc.extra.memory_accessed(ptr, size, access)
+    }
+
+    #[inline(always)]
+    fn memory_deallocated(
+        alloc: &mut Allocation<Self::PointerTag, Self::AllocExtra>,
+        ptr: Pointer<Borrow>,
+    ) -> EvalResult<'tcx> {
+        alloc.extra.memory_deallocated(ptr)
+    }
+
+    #[inline(always)]
+    fn tag_reference(
+        ecx: &mut EvalContext<'a, 'mir, 'tcx, Self>,
+        ptr: Pointer<Borrow>,
+        pointee_ty: Ty<'tcx>,
+        pointee_size: Size,
+        borrow_kind: Option<mir::BorrowKind>,
+    ) -> EvalResult<'tcx, Borrow> {
+        if !ecx.machine.validate {
+            // No tracking
+            Ok(Borrow::default())
+        } else {
+            ecx.tag_reference(ptr, pointee_ty, pointee_size, borrow_kind)
+        }
+    }
+
+    #[inline(always)]
+    fn tag_dereference(
+        ecx: &EvalContext<'a, 'mir, 'tcx, Self>,
+        ptr: Pointer<Borrow>,
+        ptr_ty: Ty<'tcx>,
+    ) -> EvalResult<'tcx, Borrow> {
+        if !ecx.machine.validate {
+            // No tracking
+            Ok(Borrow::default())
+        } else {
+            ecx.tag_dereference(ptr, ptr_ty)
+        }
+    }
 }
