@@ -25,7 +25,7 @@ use log::*;
 use std::path::{Path, PathBuf};
 
 /// Cleanup documentation code blocks. The `docs` are expected to have
-/// the preceeding `///` or `//!` prefixes already trimmed away. Rust code
+/// the preceding `///` or `//!` prefixes already trimmed away. Rust code
 /// blocks will ignore lines beginning with `#`. Code block annotations
 /// that are common to Rust will be converted to `rust` allow for markdown
 /// syntax coloring.
@@ -120,6 +120,11 @@ pub fn extract_docs(
 
         if attr_start && line.ends_with(']') && !hit_top {
             // Ignore single line attributes
+            trace!(
+                "extract_docs: ignoring single-line attribute, next_row: {:?}, up: {}",
+                next_row,
+                up
+            );
             continue;
         }
 
@@ -128,12 +133,23 @@ pub fn extract_docs(
         if attr_start || (line.ends_with(']') && !line.starts_with("//")) {
             in_meta = !in_meta;
             if !in_meta && !hit_top {
+                trace!(
+                    "extract_docs: exiting multi-line attribute, next_row: {:?}, up: {}",
+                    next_row,
+                    up
+                );
                 continue;
             };
         }
 
-        if in_meta {
+        if !hit_top && in_meta {
             // Ignore milti-line attributes
+            trace!(
+                "extract_docs: ignoring multi-line attribute, next_row: {:?}, up: {}, in_meta: {}",
+                next_row,
+                up,
+                in_meta
+            );
             continue;
         } else if line.starts_with("////") {
             trace!(
@@ -173,7 +189,9 @@ pub fn extract_docs(
             } else {
                 docs.push(doc_line);
             }
-        } else if hit_top {
+        }
+
+        if hit_top {
             // The top of the file was reached
             debug!(
                 "extract_docs: bailing out: prev_row == next_row; next_row = {:?}, up = {}",
@@ -182,14 +200,14 @@ pub fn extract_docs(
             break;
         } else if line.starts_with("//") {
             trace!(
-                "extract_docs: ignoring non-doc comment, next_row: {:?}, up: {}",
+                "extract_docs: continuing after comment line, next_row: {:?}, up: {}",
                 next_row,
                 up
             );
             continue;
         } else if line.is_empty() {
             trace!(
-                "extract_docs: ignoring empty line, next_row: {:?}, up: {}",
+                "extract_docs: continuing after empty line, next_row: {:?}, up: {}",
                 next_row,
                 up
             );
@@ -223,7 +241,7 @@ fn extract_and_process_docs(vfs: &Vfs, file: &Path, row_start: Row<ZeroIndexed>)
         .and_then(empty_to_none)
 }
 
-/// Extracts a function, method, struct, enum, or trait decleration
+/// Extracts a function, method, struct, enum, or trait declaration
 /// from source.
 pub fn extract_decl(
     vfs: &Vfs,
@@ -509,7 +527,7 @@ fn create_tooltip(
         tooltip.push(MarkedString::from_markdown(doc_url));
     }
     if let Some(context) = context {
-        tooltip.push(MarkedString::from_language_code(rust.clone(), context));
+        tooltip.push(MarkedString::from_language_code(rust, context));
     }
     if let Some(docs) = docs {
         tooltip.push(MarkedString::from_markdown(docs));
@@ -594,25 +612,35 @@ fn collapse_parents(path: PathBuf) -> PathBuf {
 /// Converts a racer `Match` to a save-analysis `Def`. Returns
 /// `None` if the coordinates are not available on the match.
 fn racer_match_to_def(ctx: &InitActionContext, m: &racer::Match) -> Option<Def> {
-    use racer::MatchType::*;
-    let kind = match m.mtype {
-        Struct | Impl | TraitImpl => DefKind::Struct,
-        Module => DefKind::Mod,
-        MatchArm => DefKind::Local,
-        Function => DefKind::Function,
-        Crate => DefKind::Mod,
-        Let | IfLet | WhileLet | For => DefKind::Local,
-        StructField => DefKind::Field,
-        Enum => DefKind::Enum,
-        EnumVariant(_) => DefKind::StructVariant,
-        Type | TypeParameter(_) => DefKind::Type,
-        FnArg => DefKind::Local,
-        Trait => DefKind::Trait,
-        Const => DefKind::Const,
-        Static => DefKind::Static,
-        Macro => DefKind::Macro,
-        Builtin => DefKind::Macro,
-    };
+    use racer::MatchType;
+    fn to_def_kind(kind: &MatchType) -> DefKind {
+        match kind {
+            MatchType::Struct(_) => DefKind::Struct,
+            MatchType::Module => DefKind::Mod,
+            MatchType::MatchArm => DefKind::Local,
+            MatchType::Function | MatchType::Method(_) => DefKind::Function,
+            MatchType::Crate => DefKind::Mod,
+            MatchType::Let(_)
+            | MatchType::IfLet(_)
+            | MatchType::WhileLet(_)
+            | MatchType::For(_) => DefKind::Local,
+            MatchType::StructField => DefKind::Field,
+            MatchType::Enum(_) => DefKind::Enum,
+            MatchType::EnumVariant(_) => DefKind::StructVariant,
+            MatchType::Type | MatchType::TypeParameter(_) | MatchType::AssocType => DefKind::Type,
+            MatchType::FnArg(_) => DefKind::Local,
+            MatchType::Trait => DefKind::Trait,
+            MatchType::Const => DefKind::Const,
+            MatchType::Static => DefKind::Static,
+            MatchType::Macro => DefKind::Macro,
+            MatchType::Builtin(_) => DefKind::Macro,
+            MatchType::UseAlias(m) => match m.mtype {
+                MatchType::UseAlias(_) => unreachable!("Nested use aliases"),
+                _ => to_def_kind(&m.mtype),
+            },
+        }
+    }
+    let kind = to_def_kind(&m.mtype);
 
     let contextstr = if kind == DefKind::Mod {
         use std::env;
@@ -842,7 +870,7 @@ fn format_method(rustfmt: Rustfmt, fmt_config: &FmtConfig, the_type: String) ->
     result.trim().into()
 }
 
-/// Builds a hover tooltip composed of the function signature or type decleration, doc URL
+/// Builds a hover tooltip composed of the function signature or type declaration, doc URL
 /// (if available in the save-analysis), source extracted documentation, and code context
 /// for local variables.
 pub fn tooltip(
@@ -907,9 +935,13 @@ pub fn tooltip(
                 DefKind::Enum | DefKind::Union | DefKind::Struct | DefKind::Trait => {
                     tooltip_struct_enum_union_trait(&ctx, &def, doc_url)
                 }
-                DefKind::Function | DefKind::Method => tooltip_function_method(&ctx, &def, doc_url),
+                DefKind::Function | DefKind::Method | DefKind::ForeignFunction => {
+                    tooltip_function_method(&ctx, &def, doc_url)
+                }
                 DefKind::Mod => tooltip_mod(&ctx, &def, doc_url),
-                DefKind::Static | DefKind::Const => tooltip_static_const_decl(&ctx, &def, doc_url),
+                DefKind::Static | DefKind::ForeignStatic | DefKind::Const => {
+                    tooltip_static_const_decl(&ctx, &def, doc_url)
+                }
                 DefKind::Type => tooltip_type(&ctx, &def, doc_url),
                 _ => {
                     debug!(
@@ -1019,6 +1051,24 @@ pub mod test {
             fs::write(path.clone(), data)
                 .map_err(|e| format!("failed to save hover test result: {:?} ({:?})", path, e))
         }
+
+        /// Returns true if data is equal to `other` relaxed so that
+        /// `MarkedString::String` in `other` need only start with self's.
+        fn has_same_data_start(&self, other: &Self) -> bool {
+            match (&self.data, &other.data) {
+                (Ok(data), Ok(them)) if data.len() == them.len() => data
+                    .iter()
+                    .zip(them.iter())
+                    .map(|(us, them)| match (us, them) {
+                        (MarkedString::String(us), MarkedString::String(them)) => {
+                            them.starts_with(us)
+                        }
+                        _ => us == them,
+                    })
+                    .all(|r| r),
+                _ => false,
+            }
+        }
     }
 
     impl Test {
@@ -1040,7 +1090,7 @@ pub mod test {
         fn run(&self, project_dir: &Path, ctx: &InitActionContext) -> TestResult {
             let url =
                 Url::from_file_path(project_dir.join("src").join(&self.file)).expect(&self.file);
-            let doc_id = TextDocumentIdentifier::new(url.clone());
+            let doc_id = TextDocumentIdentifier::new(url);
             let position = Position::new(self.line - 1u64, self.col - 1u64);
             let params = TextDocumentPositionParams::new(doc_id, position);
             let result = tooltip(&ctx, &params).map_err(|e| format!("tooltip error: {:?}", e));
@@ -1079,9 +1129,9 @@ pub mod test {
         /// Clears and returns the recorded output lines
         pub fn reset(&self) -> Vec<String> {
             let mut lines = self.lines.lock().unwrap();
-            let mut swaped = Vec::new();
-            ::std::mem::swap(&mut *lines, &mut swaped);
-            swaped
+            let mut swapped = Vec::new();
+            ::std::mem::swap(&mut *lines, &mut swapped);
+            swapped
         }
     }
 
@@ -1127,9 +1177,9 @@ pub mod test {
             let vfs = Arc::new(Vfs::new());
 
             let ctx = InitActionContext::new(
-                analysis.clone(),
-                vfs.clone(),
-                config.clone(),
+                analysis,
+                vfs,
+                config,
                 client_caps,
                 project_dir.clone(),
                 pid,
@@ -1190,7 +1240,7 @@ pub mod test {
                             if actual_result.test != expect_result.test {
                                 let e = format!("Mismatched test: {:?}", expect_result.test);
                                 Some((Err(e), actual_result))
-                            } else if actual_result == expect_result {
+                            } else if expect_result.has_same_data_start(&actual_result) {
                                 None
                             } else {
                                 Some((Ok(expect_result), actual_result))
@@ -1238,7 +1288,7 @@ pub mod test {
     }
 
     /// Strips indentation from string literals by examining
-    /// the indent of the first non-empty line. Preceeding
+    /// the indent of the first non-empty line. Preceding
     /// and trailing whitespace is also removed.
     fn noindent(text: &str) -> String {
         let indent = text
@@ -1585,7 +1635,7 @@ pub mod test {
             )
         ",
         );
-        let result = format_method(fmt.clone(), config, input);
+        let result = format_method(fmt, config, input);
         assert_eq!(expected, result, "function with multiline args");
     }
 
@@ -1597,56 +1647,56 @@ pub mod test {
         let expected = "pub fn foo() -> Foo<u32>";
         let row_start = Row::new_zero_indexed(10);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("fuction decleration")
+            .expect("function declaration")
             .join("\n");
         assert_eq!(expected, actual);
 
         let expected = "pub struct Foo<T>";
         let row_start = Row::new_zero_indexed(15);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("struct decleration")
+            .expect("struct declaration")
             .join("\n");
         assert_eq!(expected, actual);
 
         let expected = "pub enum Bar";
         let row_start = Row::new_zero_indexed(20);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("enum decleration")
+            .expect("enum declaration")
             .join("\n");
         assert_eq!(expected, actual);
 
         let expected = "pub struct NewType(pub u32, f32)";
         let row_start = Row::new_zero_indexed(25);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("tuple decleration")
+            .expect("tuple declaration")
             .join("\n");
         assert_eq!(expected, actual);
 
         let expected = "pub fn new() -> NewType";
         let row_start = Row::new_zero_indexed(28);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("struct function decleration")
+            .expect("struct function declaration")
             .join("\n");
         assert_eq!(expected, actual);
 
         let expected = "pub fn bar<T: Copy + Add>(&self, the_really_long_name_string: String, the_really_long_name_foo: Foo<T>) -> Vec<(String, Foo<T>)>";
         let row_start = Row::new_zero_indexed(32);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("long struct method decleration with generics")
+            .expect("long struct method declaration with generics")
             .join("\n");
         assert_eq!(expected, actual);
 
         let expected = "pub trait Baz<T> where T: Copy";
         let row_start = Row::new_zero_indexed(37);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("enum decleration")
+            .expect("enum declaration")
             .join("\n");
         assert_eq!(expected, actual);
 
         let expected = "fn make_copy(&self) -> Self";
         let row_start = Row::new_zero_indexed(38);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("trait method decleration")
+            .expect("trait method declaration")
             .join("\n");
         assert_eq!(expected, actual);
 
@@ -1666,7 +1716,7 @@ pub mod test {
         );
         let row_start = Row::new_zero_indexed(47);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("trait decleration multiline")
+            .expect("trait declaration multiline")
             .join("\n");
         assert_eq!(expected, actual);
 
@@ -1680,7 +1730,7 @@ pub mod test {
         );
         let row_start = Row::new_zero_indexed(53);
         let actual = extract_decl(&vfs, file, row_start)
-            .expect("function decleration multiline")
+            .expect("function declaration multiline")
             .join("\n");
         assert_eq!(expected, actual);
     }
@@ -1750,7 +1800,7 @@ pub mod test {
                 U: Clone,
         ",
         );
-        let result = format_object(fmt.clone(), config, input.into());
+        let result = format_object(fmt, config, input.into());
         assert_eq!(expected, result, "trait with where clause");
     }
 
@@ -1956,6 +2006,21 @@ pub mod test {
         assert_eq!(expected, actual);
     }
 
+    #[test]
+    fn test_extract_docs_comment_first_line() {
+        let vfs = Vfs::new();
+        let file = Path::new("test_data/hover/src/test_extract_docs_comment_first_line.rs");
+
+        let row_start = Row::new_zero_indexed(1);
+        let actual = extract_docs(&vfs, &file, row_start)
+            .expect(&format!("failed to extract docs: {:?}", file))
+            .join("\n");
+
+        let expected = "First line comment";
+
+        assert_eq!(expected, actual);
+    }
+
     #[test]
     // doesn't work in the rust-lang/rust repo, enable on CI
     #[cfg_attr(not(enable_tooltip_tests), ignore)]
