@@ -167,7 +167,7 @@ fn normalize_ranges(ranges: &mut HashMap<FileName, Vec<Range>>) {
         ranges.sort();
         let mut result = vec![];
         {
-            let mut iter = ranges.into_iter().peekable();
+            let mut iter = ranges.iter_mut().peekable();
             while let Some(next) = iter.next() {
                 let mut next = *next;
                 while let Some(&&mut peek) = iter.peek() {
@@ -216,7 +216,8 @@ impl FileLines {
                 .map(|(file, range)| JsonSpan {
                     file: file.to_owned(),
                     range: (range.lo, range.hi),
-                }).collect(),
+                })
+                .collect(),
         }
     }
 
@@ -402,9 +403,9 @@ mod test {
                 vec![Range::new(1, 7)],
             ),
         ]
-            .iter()
-            .cloned()
-            .collect();
+        .iter()
+        .cloned()
+        .collect();
 
         let file_lines = FileLines::from_ranges(ranges);
         let mut spans = file_lines.to_json_spans();
