@@ -19,8 +19,8 @@ use syntax::{ast, ptr};
 use comment::{combine_strs_with_missing_comments, rewrite_comment};
 use config::{Config, ControlBraceStyle, IndentStyle};
 use expr::{
-    format_expr, is_empty_block, is_simple_block, is_unsafe_block, prefer_next_line,
-    rewrite_multiple_patterns, ExprType, RhsTactics, ToExpr,
+    format_expr, is_empty_block, is_simple_block, is_unsafe_block, prefer_next_line, rewrite_cond,
+    rewrite_multiple_patterns, ExprType, RhsTactics,
 };
 use lists::{itemize_list, write_list, ListFormatting};
 use rewrite::{Rewrite, RewriteContext};
@@ -231,7 +231,7 @@ fn rewrite_match_arm(
 ) -> Option<String> {
     let (missing_span, attrs_str) = if !arm.attrs.is_empty() {
         if contains_skip(&arm.attrs) {
-            let (_, body) = flatten_arm_body(context, &arm.body);
+            let (_, body) = flatten_arm_body(context, &arm.body, None);
             // `arm.span()` does not include trailing comma, add it manually.
             return Some(format!(
                 "{}{}",
@@ -247,54 +247,44 @@ fn rewrite_match_arm(
     } else {
         (mk_sp(arm.span().lo(), arm.span().lo()), String::new())
     };
-    let pats_str =
-        rewrite_match_pattern(context, &ptr_vec_to_ref_vec(&arm.pats), &arm.guard, shape)
-            .and_then(|pats_str| {
-                combine_strs_with_missing_comments(
-                    context,
-                    &attrs_str,
-                    &pats_str,
-                    missing_span,
-                    shape,
-                    false,
-                )
-            })?;
+
+    // Patterns
+    // 5 = ` => {`
+    let pat_shape = shape.sub_width(5)?;
+    let pats_str = rewrite_multiple_patterns(context, &ptr_vec_to_ref_vec(&arm.pats), pat_shape)?;
+
+    // Guard
+    let block_like_pat = trimmed_last_line_width(&pats_str) <= context.config.tab_spaces();
+    let new_line_guard = pats_str.contains('\n') && !block_like_pat;
+    let guard_str = rewrite_guard(
+        context,
+        &arm.guard,
+        shape,
+        trimmed_last_line_width(&pats_str),
+        new_line_guard,
+    )?;
+
+    let lhs_str = combine_strs_with_missing_comments(
+        context,
+        &attrs_str,
+        &format!("{}{}", pats_str, guard_str),
+        missing_span,
+        shape,
+        false,
+    )?;
 
     let arrow_span = mk_sp(arm.pats.last().unwrap().span.hi(), arm.body.span.lo());
     rewrite_match_body(
         context,
         &arm.body,
-        &pats_str,
+        &lhs_str,
         shape,
-        arm.guard.is_some(),
+        guard_str.contains('\n'),
         arrow_span,
         is_last,
     )
 }
 
-fn rewrite_match_pattern(
-    context: &RewriteContext,
-    pats: &[&ast::Pat],
-    guard: &Option<ptr::P<ast::Expr>>,
-    shape: Shape,
-) -> Option<String> {
-    // Patterns
-    // 5 = ` => {`
-    let pat_shape = shape.sub_width(5)?;
-    let pats_str = rewrite_multiple_patterns(context, pats, pat_shape)?;
-
-    // Guard
-    let guard_str = rewrite_guard(
-        context,
-        guard,
-        shape,
-        trimmed_last_line_width(&pats_str),
-        pats_str.contains("\n"),
-    )?;
-
-    Some(format!("{}{}", pats_str, guard_str))
-}
-
 fn block_can_be_flattened<'a>(
     context: &RewriteContext,
     expr: &'a ast::Expr,
@@ -313,24 +303,33 @@ fn block_can_be_flattened<'a>(
 // (extend, body)
 // @extend: true if the arm body can be put next to `=>`
 // @body: flattened body, if the body is block with a single expression
-fn flatten_arm_body<'a>(context: &'a RewriteContext, body: &'a ast::Expr) -> (bool, &'a ast::Expr) {
+fn flatten_arm_body<'a>(
+    context: &'a RewriteContext,
+    body: &'a ast::Expr,
+    opt_shape: Option<Shape>,
+) -> (bool, &'a ast::Expr) {
+    let can_extend =
+        |expr| !context.config.force_multiline_blocks() && can_flatten_block_around_this(expr);
+
     if let Some(ref block) = block_can_be_flattened(context, body) {
         if let ast::StmtKind::Expr(ref expr) = block.stmts[0].node {
             if let ast::ExprKind::Block(..) = expr.node {
-                flatten_arm_body(context, expr)
+                flatten_arm_body(context, expr, None)
             } else {
-                let can_extend_expr =
-                    !context.config.force_multiline_blocks() && can_flatten_block_around_this(expr);
-                (can_extend_expr, &*expr)
+                let cond_becomes_muti_line = opt_shape
+                    .and_then(|shape| rewrite_cond(context, expr, shape))
+                    .map_or(false, |cond| cond.contains('\n'));
+                if cond_becomes_muti_line {
+                    (false, &*body)
+                } else {
+                    (can_extend(expr), &*expr)
+                }
             }
         } else {
             (false, &*body)
         }
     } else {
-        (
-            !context.config.force_multiline_blocks() && body.can_be_overflowed(context, 1),
-            &*body,
-        )
+        (can_extend(body), &*body)
     }
 }
 
@@ -343,7 +342,11 @@ fn rewrite_match_body(
     arrow_span: Span,
     is_last: bool,
 ) -> Option<String> {
-    let (extend, body) = flatten_arm_body(context, body);
+    let (extend, body) = flatten_arm_body(
+        context,
+        body,
+        shape.offset_left(extra_offset(pats_str, shape) + 4),
+    );
     let (is_block, is_empty_block) = if let ast::ExprKind::Block(ref block, _) = body.node {
         (
             true,
@@ -376,7 +379,9 @@ fn rewrite_match_body(
     // Look for comments between `=>` and the start of the body.
     let arrow_comment = {
         let arrow_snippet = context.snippet(arrow_span).trim();
-        let arrow_index = arrow_snippet.find("=>").unwrap();
+        // search for the arrow starting from the end of the snippet since there may be a match
+        // expression within the guard
+        let arrow_index = arrow_snippet.rfind("=>").unwrap();
         // 2 = `=>`
         let comment_str = arrow_snippet[arrow_index + 2..].trim();
         if comment_str.is_empty() {
@@ -484,10 +489,18 @@ fn rewrite_match_body(
     }
 }
 
+impl Rewrite for ast::Guard {
+    fn rewrite(&self, context: &RewriteContext, shape: Shape) -> Option<String> {
+        match self {
+            ast::Guard::If(ref expr) => expr.rewrite(context, shape),
+        }
+    }
+}
+
 // The `if ...` guard on a match arm.
 fn rewrite_guard(
     context: &RewriteContext,
-    guard: &Option<ptr::P<ast::Expr>>,
+    guard: &Option<ast::Guard>,
     shape: Shape,
     // The amount of space used up on this line for the pattern in
     // the arm (excludes offset).
@@ -538,9 +551,9 @@ fn nop_block_collapse(block_str: Option<String>, budget: usize) -> Option<String
             && budget >= 2
             && (block_str[1..].find(|c: char| !c.is_whitespace()).unwrap() == block_str.len() - 2)
         {
-            "{}".to_owned()
+            String::from("{}")
         } else {
-            block_str.to_owned()
+            block_str
         }
     })
 }
