@@ -19,3 +19,10 @@ pub extern "C" fn Java_com_exonum_binding_storage_indices_ValueSetIndexProxy_nat
 ) -> bool {
     false
 }
+
+// #3009
+impl Something {
+    fn my_function_name_is_way_to_long_but_used_as_a_case_study_or_an_example_its_fine(
+    ) -> Result<(), String> {
+    }
+}
